function [Bidx,dist,Nf,abs_dist]=neighbour_search_v2(x,xB,tree,rss,omega,omg2,d,debug_on)
% searching the neighbours of a particle in a 2D setup
% the A particle is located in x and we search for B particles 
%  within distance rss
% xB = The B particles location (two column matrix)
% rss = half-lengths of the searched hypercube 
%   if d=1, rss is a scalar, and it describes the search radius.
%   if d=2, rss is a vector of size 2x1, it describes the half-width and half-height of the search rectangle
%   if d=3, rss is a vector of size 3x1, it describes the half-widths 
%       of the search cube
% omg2 = half the domain size, omega
% NOTE: the algorythm assumes that the domain is between -omg2 to +omg2 (in
% all directions)
%
% output: 
%   Bidx - indices of the neighbouring B particles (row vector)
%   dist - the distance in each direction (with periodic boundary conditions taken into
%          account) size: Nf X d
%   Nf - number of neighbours found (=length(Bidx))

if debug_on
    if any(abs(x)>omg2) || any (abs(xB)>omg2)
        error ('x should be bounded by +-omega/2')
    end

    if length(rss)~=d || length(x)~=d
        error('dimension mismatch')
    end
end
    if (any(rss>omg2))
        error('this algorythm handles search within domain less then the domain size')
    end
    
    %initialize xbnd
    xbnd = NaN(d,4); % the range min and max in the specific dimension
    dist_correct=NaN(d,2); % the correction for distance (based on the periodic b.c.) 
    zones=ones(d,1);
    for i=1:d
        xbnd(i,1:2)=[x(i)-rss(i) x(i)+rss(i)]; 
        dist_correct(i,1)=0;
        % add "the other side", if needed
        if x(i)+rss(i)> omg2
            xbnd(i,2:4)=   [     omg2        -omg2       -omega+x(i)+rss(i)];
            dist_correct(i,2) =omega;
            zones(i)=2;
        end
        if x(i)-rss(i)<-omg2
            xbnd(i,1:4)=[-omg2  x(i)+rss(i)  omega+x(i)-rss(i)   omg2];
            dist_correct(i,2)= -omega;
            zones(i)=2;
        end 
    end      

    % N=size(xB,1); %number of B particles
    % start_length is an initial (guessed) length for
    %the vector that holds the indices of neighbours
    % factor=1.3; % we want to have some extra space in advance, so factor is slightly bigger then 1
    % start_length=ceil(N*factor^d*prod(rss)/omg2^d); 
    % idx=zeros(start_length,1); % a vector that contain the indices to all particles found

    [Bidx,dist,Nf]=recursive_range_extract(tree,x,xB,d,xbnd,dist_correct,zones);
    % to get absolute distance, sum along rows
    abs_dist=sqrt(sum(dist.^2,2));        
           
        
function [idx,dist,Nf]=recursive_range_extract(tree,x,xB,d,xbnd,dist_correct,zones)
% zones = column vector (num. elements = d). zones(i)=1,2 and tells how
% many zones exist in the i-th dimension
% this function will run prod(zones) times to take account of all possible
% search volumes (whose maximum number is 2^d)
% tree - kdtree of B particle locations
% xbnd - boundaries of search zones, dX4.
% for example xbnd=[-1 -0.9 0.95 1] means we need to search at [-1 -0.9]
% and [0.95 1]. this is a 1D example.
% idx,dist are initially empty and feel up with the indices and respective
% corrected distance to particles

flag=0;
for i=1:d
    if zones(i)==2
        flag=1;
        % break zones into two and reiterate function for each
        % then merge results and make them the output
        
        % first, set temporary variables with the i-th dimension parameter
        %    split between them. xbnd is split into xbnd1 and xbnd2
        % and same for dist_correct
        

        xbnd1=xbnd; xbnd1(i,3:4)=NaN;
        xbnd2=xbnd; xbnd2(i,1:2)=xbnd(i,3:4); xbnd2(i,3:4)=NaN;
        
        dist_correct1=dist_correct; dist_correct1(i,2)=NaN;
        dist_correct2=dist_correct; dist_correct2(i,1:2)=[dist_correct(i,2) NaN];
        
        % now, zones needs to be just 1 in the i-th dimension
        zones1=zones; zones1(i)=1; 
        zones2=zones1;
                
        [idx1,dist1,Nf1]=recursive_range_extract(tree,x,xB,d,xbnd1,dist_correct1,zones1);
        [idx2,dist2,Nf2]=recursive_range_extract(tree,x,xB,d,xbnd2,dist_correct2,zones2);
        % now merge results
        idx=[idx1;idx2];
        dist=[dist1; dist2];
        Nf=Nf1+Nf2;
        
        break
    end
end
if flag==0 % zones contains only 1
    %find neighbours and make them the output
    range=xbnd(:,1:2);
%     [idx,xBf]= kdtree_range(tree, range); 
    % xBf = location of points found(size Nf X d)
    % where Nf = number of found particles
    % idx = indices of the found particles (size 1 X Nf)
    midpoint=mean(range,2); % take mean along rows
    r=max(abs(range(:,2)-range(:,1)))/2;
    % the radius of the circle that contains the search ellipse
    Bidx = rangesearch(tree,midpoint',r);
    idx=(Bidx{1})';      % change cell to row vector, then rotate to column vector (Nf X 1)
    xBf=xB(idx,:);
    Nf=size(xBf,1);
    dist=zeros(Nf,d);   % distance between x to B particles.(size Nf X d)   
    if ~isempty(idx)
        for i=1:d
            x1=x(i)-dist_correct(i,1); % position of the (virtual) particle in i-th dimension

            dist(:,i)=x1-xBf(:,i); % the distance between the B particles and that virtual A particle
        end
    end
end