function plot_concX_v2(xA,mpA,C_0,Kf,omega,D,alpha,dt1,t1,d,fig_num)
% calculate and plot the conc. map of A
% fig_num - number of figure  to plot on 
% if fig_num=0, keep plotting on current figure, but use different color
% (for plotting C_A and C_B on the same graph)
% xA - locations of particles (matrix of size [NA,d])
% mpA- masses of particles (column vecotr, length NA)
% t1 - current simulation time
% dt1 - current simulation time step size
% d - dimension [=1,2,3]

[NA,d1]=size(xA);
if d1~=d
    error('paricle location dimension does not match with d')
end
if NA~=length(mpA)
    error('mpA is not matching xA')
end

color_str='b';

if fig_num==0
    hold on
    color_str='r';
else
    figure(fig_num)
end

bins=1000;
x=linspace(0,omega,bins); % dx is omega/(bins-1)
sizeC=repmat(bins,1,d);
C=calculate_conc(xA,mpA,omega,D,alpha,dt1,d,bins,sizeC);
switch d
    case 1
        plot(x/omega,C/C_0,color_str)
        ylabel('C/C_0')
    case 2
        imagesc(x/omega,x/omega,C/C_0)
        set(gca,'ydir','normal')
        ylabel('y/\Omega')
        axis equal tight 
        colorbar

    case 3
        [X,Y,Z]=meshgrid(x,x,x);
        xslice = [1/3, 2/3]; yslice = 1/2; 
        zslice = [1/3, 2/3];
        slice(X/omega,Y/omega,Z/omega,C/C_0,xslice,yslice,zslice)
        colormap hsv
end
title (['C*(x*,t*=' num2str(t1*Kf*C_0,'%05.2f') ')'])
xlabel('x/\Omega')

if fig_num==0
    hold off
end

function C=calculate_conc(xA,mpA,omega,D,alpha,dt,d,bins,sizeC)
C=zeros(sizeC);
% auxiliary variables
kappa1=kappa(alpha,D,dt,d);
ki=kappa1^(-1); % the inverse of kappa 
sdk=sqrt (det(kappa1));
factor=1./((2*pi)^(d/2)*sdk);
a=12;
r2=2*a*(det(kappa1))^(1/d); 
r=sqrt(r2); % the approximate radius where the relative concentration is exp(-a), i.e. negligible
ri=round(r*(bins-1))+1;
if r>omega/5
    error('r, the support scale of particle must be smaller than omega/5')
end
% add imaginary particles next to boundaries (like padding)
for i=1:d
    xfl=find(xA(:,i)<r);
    xA_imag=xA(xfl,:);
    xA_imag(:,i)=xA_imag(:,i)+omega;

    xfh=find(xA(:,i)>(omega-r));
    xA_imag1=xA(xfh,:);
    xA_imag1(:,i)=xA_imag1(:,i)-omega;
    
    % add those imaginary particles
    xA=[xA;xA_imag;xA_imag1];
    mpA=[mpA;mpA(xfl);mpA(xfh)];
end    

dx=omega/(bins-1);
N=size(xA,1); %number of particles, including imaginary ones

switch d
    case 1
        CMx=(-ri:ri)*dx;
        denom=2*kappa1;
        CM=factor*exp(-CMx.^2/denom);
    case 2
        [CMx,CMy]=meshgrid((-ri:ri)*dx,(-ri:ri)*dx);
        ki11=ki(1,1); ki12=ki(1,2); ki22=ki(2,2);
        F=ki11*CMx.^2 + 2*ki12*CMx.*CMy + ki22*CMy.^2;
        CM=factor*exp(-F/2);
end
CM=CM/(sum(CM(:))*dx^d); % correct so that it is normalized, and sums to 1

for i=1:N
    xA1=xA(i,:); % current A particle location
    mp1=mpA(i); % and its mass
    xAi=round(xA1/omega*(bins-1))+1; % the indices in C matrix of this location
    % the b(begin) index of the range in C to be updated by the mass of
    % this B particle
    xb=xAi-ri; 
    % correct for the min and max value (1..bins)
    xb=max(1,xb); xb=min(bins,xb); 
    % the begin indices for the CM matrix
    mxb= ones(1,d) +xb-(xAi-ri);
    % the end index
    xe=xAi+ri; 
    xe=max(1,xe); xe=min(bins,xe); 
    % the end indices in the CM matrix
    mxe=(ones(1,d)+2*ri)+xe-(xAi+ri);
    
    if all(xe>=xb) % make sure there's something left (to prevent error)
         C=addC(C,xb,xe,mxb,mxe,mp1,CM,d);
    end
end
if d==2
    C=C'; %transpose !
end

function C=addC(C,xb,xe,mxb,mxe,mp1,CM,d)
switch d
   case 1
%        try
        C(xb(1):xe(1))=C(xb(1):xe(1)) + mp1* CM(mxb(1):mxe(1));
%        catch 
%            keyboard
%        end
   case 2
        C(xb(1):xe(1) , xb(2):xe(2))=...
            C(xb(1):xe(1) ,  xb(2):xe(2)) + ...
               mp1* CM(mxb(1):mxe(1),mxb(2):mxe(2));
end
        

