% single realization of a particle system
% particles move by brownian motion and react 
function [c,sumpr]=rw_realization(av,d,Nsim,D,Kf,C_0,omega,N_0,alpha,nos,t,dt,mp0, sim,seed_val)
% parameters:
%     av is Administritive Variables struct
%     d= number of dimensions. can be 1,2,3
%     Nsim = number of simulations (realizations)
%     D = dispersion coefficient
%     Kf = reaction rate
%     C_0 = initial concentration for both species (A,B)
%     NA_0 = initial number of A particles
%     NB_0 = initial number of B particles
%     alpha =  shear rate constant 
%     T =  maximum value of time to be reached
%     t_mult = multiplier of time step
%     dt0 = initial time step
%     max_dt = maximum time step

fprintf('\n Starting Simulation number %d of %d...\n',sim,Nsim)
rng(seed_val); % Reinitialize the random number generator with the random seed
%start the time counters
elapsed_time0=now; 
elapsed_timeTBDM=now;
elapsed_timeTBP=now;
%% basic definitions
TBDM=av.TBDM/86400 ;  %time [day] between displaying number of step
TBP=av.TBP/86400; %time [day] between plotting concentration distribution
start_at=1;  % time step to start at
n_previous=start_at;  % initializing n_previous (later used to calculate the time left for simulation process)
% initalizing location of particles by a random distribution:   the 
% particles get a random position at [0 omega]^d
NA_0=N_0;
NB_0=N_0;
xA=rand(NA_0,d)*omega;
xB=rand(NB_0,d)*omega; 
%the intial mass of the particels
mpA=ones(NA_0,1)*mp0; 
mpB=ones(NB_0,1)*mp0;
% allocating space for vector of average concentration
c=zeros(nos+1,1);
c(1)=C_0; % average concentration at t=0 is C_0. That's why we need nos+1 spaces in c
sumpr=zeros(nos,1);
%% next, start the simulation
for n=1:nos % counter  of time steps
    %% general definitions 
    dt1=dt(n);  % current time step size
    t1=t(n);    % current time(beginning of time step)
    NA=size(xA,1); 
    NB=size(xB,1);
    %% save locations and mass of particles
    if any(n==av.tstbs)&& sim==1 % save only time steps "tstbs", in simulation #1
       try
          fname=[av.sim_folder 'part_loc' num2str(sim) '_tsi=' num2str(n) '.mat']; 
                                        %tsi = Time Step Index
          save (fname,'xA','xB','mpA','mpB','t1') % note that t1 is dimensional
          disp(['saved file ' fname ])
       catch
          disp(['error occured. Unable to save file ' fname])
       end
    end
    %% plot particles concentrations
    tmpnow=now;
    if ( tmpnow-elapsed_timeTBP)>TBP % time between plots
        if  av.plot_conc_now
           fig_num=1;
           % disp('plotting C(x)...')
           % plot the A concentration first
           plot_concX_v2(xA,mpA,C_0,Kf,omega,D,alpha,dt1,t1,d,fig_num)
           if av.save_conc_plot
               fname1=[av.fig_folder 'img' num2str(n,'%04i') '.png'];
               saveas(gcf,fname1)
               disp(['saved figure in ' fname1])
           end
           if  av.plot_conc_nowB
               if d==1, fig_num=0; else fig_num=3; end
               % plot the B distribution in the same fig (if d=1) or in fig 3
               % (otherwise)
%                plot_concX_v1(xB,mpB,C_0,Kf,omega,t(n+1),d,h,fig_num)
               plot_concX_v2(xB,mpB,C_0,Kf,omega,D,alpha,dt1,t1,d,fig_num)
           end
        end           
        TBP=TBP*1.01; % every step, we have a bit larger time difference between plotting                  
        elapsed_timeTBP=tmpnow; % set the elapsed time 
    end
    %% NOW, walk and react
    [mpA,mpB,xA,xB,sumpr]=recursive_walk_and_react(alpha,...
                    D,dt1,d,Kf,omega,av,xA,xB,NA,NB,mpA,mpB,n,sumpr);
    c(n+1)=sum(mpA)/omega^d; %the average concentration at the end of the time step
    %% display message in Command Window
    if ( tmpnow-elapsed_timeTBDM)>TBDM % time between displaying messages
        disp(['Sim #' num2str(sim) ': time step ' num2str(n) ' out of estimated number of ' num2str(nos) ])
        if n>1
            disp(['Approximate run end at ' datestr(tmpnow+(nos-n)/(n-n_previous)*(tmpnow-elapsed_timeTBDM))])
        end
        TBDM=TBDM*1.2; %  every step, we have a bit larger time difference between displaying message to user
        elapsed_timeTBDM=tmpnow; % set the elapsed time 
        n_previous=n; %keep the current step number for future calculation of estimated time left to run        
    end
end % for n=1:nos
% display run time
t_run=now-elapsed_time0;
disp(['Realization # ' num2str(sim) ': run ended at ' datestr(now) ...
    '. This realization took ' num2str(t_run*86400) ' seconds to run.'])
